@@ -5,6 +5,7 @@ import datetime as dt
 import doctest
 import json
 import pickle
+import re
 import tempfile
 import unittest
 from decimal import Decimal
@@ -12,6 +13,8 @@ from io import BytesIO, StringIO
 from pathlib import Path
 from uuid import uuid4
 
+import xlrd
+from odf import opendocument, table
 from openpyxl.reader.excel import load_workbook
 
 import tablib
@@ -1165,6 +1168,16 @@ class TSVTests(BaseTestCase):
 
 
 class ODSTests(BaseTestCase):
+    FORMAT_CONVERT = {
+        'yearlong': '%Y',
+        'monthlong': '%m',
+        'daylong': '%d',
+        'hourslong': '%H',
+        'minuteslong': '%M',
+        'secondslong': '%S',
+        'secondslong0': '%S',
+    }
+
     def test_ods_export_import_set(self):
         date = dt.date(2019, 10, 4)
         date_time = dt.datetime(2019, 10, 4, 12, 30, 8)
@@ -1185,6 +1198,39 @@ class ODSTests(BaseTestCase):
         self.assertEqual(data.dict[0]['date/time'], date_time)
         self.assertEqual(data.dict[0]['None'], '')
 
+    def test_ods_export_display(self):
+        """Test that exported datetime types are displayed correctly in office software"""
+        date = dt.date(2019, 10, 4)
+        date_time = dt.datetime(2019, 10, 4, 12, 30, 8)
+        time = dt.time(14, 30)
+        data.append((date, time, date_time))
+        data.headers = ('date', 'time', 'date/time')
+        _ods = data.ods
+        ods_book = opendocument.load(BytesIO(_ods))
+        styles = {style.getAttribute('name'): style for style in ods_book.styles.childNodes}
+        automatic_styles = {
+            style.getAttribute('name'): style.getAttribute('datastylename')
+            for style in ods_book.automaticstyles.childNodes
+        }
+
+        def get_format(cell):
+            style = styles[automatic_styles[cell.getAttribute('stylename')]]
+            f = []
+            for number in style.childNodes:
+                name = number.qname[1] + ''.join(number.attributes.values())
+                f.append(self.FORMAT_CONVERT.get(name, str(number)))
+            return ''.join(f)
+
+        cells = ods_book.spreadsheet.getElementsByType(table.TableRow)[1].childNodes
+        self.assertEqual(str(date), str(cells[0]))
+        self.assertEqual('%Y-%m-%d', get_format(cells[0]))
+
+        self.assertEqual(str(time), str(cells[1]))
+        self.assertEqual('%H:%M:%S', get_format(cells[1]))
+
+        self.assertEqual(str(date_time), str(cells[2]))
+        self.assertEqual('%Y-%m-%d %H:%M:%S', get_format(cells[2]))
+
     def test_ods_import_book(self):
         ods_source = Path(__file__).parent / 'files' / 'book.ods'
         with ods_source.open('rb') as fh:
@@ -1213,6 +1259,26 @@ class ODSTests(BaseTestCase):
             dataset = tablib.Dataset().load(fh, 'ods')
         self.assertEqual(dataset.pop(), ('abcd',))
 
+    def test_ods_export_dates(self):
+        """test against odf specification"""
+        date = dt.date(2019, 10, 4)
+        date_time = dt.datetime(2019, 10, 4, 12, 30, 8)
+        time = dt.time(14, 30)
+        data.append((date, time, date_time))
+        data.headers = ('date', 'time', 'date/time')
+        _ods = data.ods
+        ods_book = opendocument.load(BytesIO(_ods))
+        cells = ods_book.spreadsheet.getElementsByType(table.TableRow)[1].childNodes
+        # date value
+        self.assertEqual(cells[0].getAttribute('datevalue'), '2019-10-04')
+        # time value
+        duration_exp = re.compile(r"^P(?:(\d+)Y)?(?:(\d+)M)?(?:(\d+)D)?"
+                                  r"(?:T(?:(\d+)H)?(?:(\d+)M)?(?:([\d.]+)S)?)?$")
+        duration = duration_exp.match(cells[1].getAttribute('timevalue')).groups()
+        self.assertListEqual([0, 0, 0, 14, 30, 0], [int(v or 0) for v in duration])
+        # datetime value
+        self.assertEqual(cells[2].getAttribute('datevalue'), '2019-10-04T12:30:08')
+
 
 class XLSTests(BaseTestCase):
     def test_xls_format_detect(self):
@@ -1254,8 +1320,44 @@ class XLSTests(BaseTestCase):
         book = tablib.Databook().load(in_stream, 'xls')
         self.assertEqual(book.sheets()[0].title, 'Founders')
 
+    def test_xls_export_with_dates(self):
+        date = dt.date(2019, 10, 4)
+        time = dt.time(14, 30)
+        date_time = dt.datetime(2019, 10, 4, 12, 30, 8)
+        data.append((date, time, date_time))
+        data.headers = ('date', 'time', 'date/time')
+        _xls = data.xls
+        xls_book = xlrd.open_workbook(file_contents=_xls, formatting_info=True)
+        row = xls_book.sheet_by_index(0).row(1)
+
+        def get_format_str(cell):
+            return xls_book.format_map[xls_book.xf_list[cell.xf_index].format_key].format_str
+
+        self.assertEqual('m/d/yy', get_format_str(row[0]))
+        self.assertEqual('h:mm:ss', get_format_str(row[1]))
+        self.assertEqual('m/d/yy h:mm', get_format_str(row[2]))
+
 
 class XLSXTests(BaseTestCase):
+    def _helper_export_column_width(self, column_width):
+        """check that column width adapts to value length"""
+        def _get_width(data, input_arg):
+            xlsx_content = data.export('xlsx', column_width=input_arg)
+            wb = load_workbook(filename=BytesIO(xlsx_content))
+            ws = wb.active
+            return ws.column_dimensions['A'].width
+
+        xls_source = Path(__file__).parent / 'files' / 'xlsx_cell_values.xlsx'
+        with xls_source.open('rb') as fh:
+            data = tablib.Dataset().load(fh)
+        width_before = _get_width(data, column_width)
+        data.append([
+            'verylongvalue-verylongvalue-verylongvalue-verylongvalue-'
+            'verylongvalue-verylongvalue-verylongvalue-verylongvalue',
+        ])
+        width_after = _get_width(data, width_before)
+        return width_before, width_after
+
     def test_xlsx_format_detect(self):
         """Test the XLSX format detection."""
         in_stream = self.founders.xlsx
@@ -1400,6 +1502,29 @@ class XLSXTests(BaseTestCase):
         wb = load_workbook(filename=BytesIO(_xlsx))
         self.assertEqual('[1]', wb.active['A1'].value)
 
+    def test_xlsx_column_width_adaptive(self):
+        """ Test that column width adapts to value length"""
+        width_before, width_after = self._helper_export_column_width("adaptive")
+        self.assertEqual(width_before, 11)
+        self.assertEqual(width_after, 11)
+
+    def test_xlsx_column_width_integer(self):
+        """Test that column width changes to integer length"""
+        width_before, width_after = self._helper_export_column_width(10)
+        self.assertEqual(width_before, 10)
+        self.assertEqual(width_after, 10)
+
+    def test_xlsx_column_width_none(self):
+        """Test that column width does not change"""
+        width_before, width_after = self._helper_export_column_width(None)
+        self.assertEqual(width_before, 13)
+        self.assertEqual(width_after, 13)
+
+    def test_xlsx_column_width_value_error(self):
+        """Raise ValueError if column_width is not a valid input"""
+        with self.assertRaises(ValueError):
+            self._helper_export_column_width("invalid input")
+
 
 class JSONTests(BaseTestCase):
     def test_json_format_detect(self):
